#ifndef __GAMEPF_H
#define __GAMEPF_H

#include <unistd.h>

#include "definitions.h"

#include "sprite.h"

#include "ai.h"

// theme engine
#include "pax.h"

class GamePF
{
	public:
	GamePF () ;
	
	void Init ( char *side, Pax *new_pax, char *new_level = NULL, int ai_level = 0 ) ;

	int PlayerScoreGet () ;
	void PlayerScoreSet ( int value ) ;
	void PlayerScoreReset () ;
	void GemQueueAddToIncoming   ( char gem_color, int count = 1 ) ;
	void GemQueueAddToOutgoing   ( char gem_color, int count = 1 ) ;
	int  GemQueueGetFromOutgoing ( char gem_color, bool clear_after_query = FALSE ) ;
	void GemQueueInit () ;
	void GemQueueQuit () ;

	void EnableFallingGems () ;
	void DisableFallingGems () ;
	
	void Draw () ;
	
	void MoveUserGemUp ( bool force = FALSE ) ; // needed for idle counter...
	void MoveUserGemLeft () ;
	void MoveUserGemRight () ;

	void MoveUserGemUpStart () ;
	void MoveUserGemLeftStart () ;
	void MoveUserGemRightStart () ;
	void MoveUserGemUpStop () ;
	void MoveUserGemLeftStop () ;
	void MoveUserGemRightStop () ;
	void MoveUserGemToTop ( bool move_satellite = false ) ;
	void RotateSlaveGem () ;
	
	void ShootGemTo ( int cur_gem_id, int pos, int velocity = AUTO, int speed = 4 ) ;
	int  AddFallingGems ( char gem_color, int count = -1 ) ;

	void Pause () ;
	void UnPause () ;
	
	bool PlayfieldLayoutIsSet () ;
	bool PlayfieldIsSet () ;
	void SetPlayfieldIsReady ( bool choice ) ;

	void DumpStatus () ;

	bool GetIsQuitting () ;
	bool GetIsDone () ;
	bool GetIsFreed () ;
	
	void AutoLoadState () ;
	void AutoSaveState () ;
	
	void Quit () ;
	
	void Free () ;
	
	private:
	char *playfield_side ;
	Pax *pax ;
	
	int GEM_COUNT, GEM_WIDTH, GEM_HEIGHT ;
	char GEM_FIRST_COLOR, GEM_LAST_COLOR, GEM_BLOCK, GEM_HOLD, GEM_NONE ;
	
	int preview_gems ;
	
	Audio_Sound playerscore_inc_sound, playerscore_dec_sound, playerlose_sound ;
	Audio_Sound playerscore_bonus_sound ;
	KrTextBox *playernamebox, *playerscorebox, *playercombobox ;
	int playerscore_combo ;
	int playerscore_combo_rest_seconds, playerscore_combo_max_rest_seconds ;
	int playerscore_cur, playerscore_dest ;
	int playerscoreupdatetimer, playerscoreupdatecounter ;
	void PlayerScoreUpdate () ;
	bool PlayerScoreIsInComboTime () ;
	int *gems_incoming, *gems_outgoing ;
	int *gems_incoming_timer ;
	Sprite **gems_incoming_preview ;
	KrResourceVault *gems_incoming_preview_text_vault ;
	KrFontResource *gems_incoming_preview_text_font ;
	KrTextBox **gems_incoming_preview_text ;
	int updategemqueuetimer ;
	void GemQueueUpdate () ;

	char *GetPlayfieldLayout ( char *level_name ) ;
	void SetupPlayfield () ;
	int GetMatchingGem ( int dest_pos ) ; // get gem id on pos
	void HighlightEqualGems () ; // let gems similar to user's glow
	bool SlidingFinishedRow ( int row ) ;
	bool SlidingFinished () ;
	
	int GetRelPos ( int pos_in_playfield, int x, int y ) ;
	int CheckGemForFriends ( int pos_in_playfield ) ;
	int GetUnusedGem () ;
	
	struct gem_color_data
	{
		char name   [ 256 ] ; // something like block
		char string [ 256 ] ; // something like gem_block_color
		U8   exp_red   ; // gem explosion color ( red part )
		U8   exp_green ; // gem explosion color ( green part )
		U8   exp_blue  ; // gem explosion color ( blue part )
	} ;
	
	gem_color_data *GetGemColorDataFromPlayfieldChar ( char color ) ;
	int SetupGem ( char gem_color,
					int pos_in_playfield,
					KrImNode *field,
					int x,
					int y,
					bool do_slide,
					int slide_x,
					int slide_y ) ;
	void UpdateGemPreview () ;
	int GetEmptyPosAtBottom () ;
	void CreateUserGemCursorAtPos ( int pos ) ;
	void ThrowInGem ( int pos = -1, bool do_slide = true ) ;
	void DestroyPlayfield () ;
	void CheckForFullPlayfield () ;
	void FreezeGem ( int id ) ;
	void UnfreezeGem ( int id ) ;
	void UnfreezeGemArea ( int id ) ;

	KrEngine *engine ;
	bool using_opengl ;

	char *level ; // which level do we play? linked to playfield-layout
	bool playfield_is_set_up ;
	int playfield_throw_in_effect ;
	
	KrResourceVault *Vault ;
	KrImNode *basefield ;
	KrImNode *previewfield ;
	KrImNode *incomingfield ;

	int PLAYFIELD_MAX_X, PLAYFIELD_MAX_Y, PLAYFIELD_SIZE ;

	char *playfield ;
	Sprite **gem ;
	Sprite **gem_frozen_layer ;
	bool *gem_active ;
	int *gem_pos ;
	bool *gem_frozen ;
	
	bool incoming_earthquake ; // an earthquake occurs when to many gems move
	Audio_Sound earthquake_sound ;
	Audio_Sound pre_earthquake_sound ;
	
	int pull_row ; // when setting up our playfield which row to pull in
	int pull_row_counter ; // to setup a little delay
	
	// collection of connected gems on playfield
	GlDynArray < int > *friend_bucket ;
	
	int action_indicator ;
	int action_step_one ;
	int action_step_two ;
	int action_max ;
	
	int auto_play ; // whether user can handle the gems
	int user_gem_id ; // the gems which are handeled by the user
	int slave_gem_id ; // the satellite around the user gem :)
	bool slave_gem_thrown ; // whether a slave was already thrown this time
	Sprite *user_gem_cursor, *slave_gem_cursor ; // cursor overlays
	int *preview_gem_id ;
	char *preview_gem_color ;
	int move_on_counter ;
	int move_on_delay ;
	bool move_up ;
	bool move_left ;
	bool move_right ;
	
	bool idle_rows_enabled ; // whether to push a row into here and then...
	int idle_row_counter ;
	int idle_row_delay ;
	
	bool destroyable_gems ;
	
	bool falling_gems_enabled ; // whether gems should fall up the screen
	bool freeze_gems_at_start ; // whether to freeze all gems at game-start
	
	int gem_colors_activated ; // how many colors to be used ( max = GEM_COUNT ! )
	
	AI Ai ; // artificial intelligence :)
	
	bool playfield_set ; // whether playfield is setup for playing
	bool playfield_ready ; // whether player can play or not
	
	//
	int GetData ( char *message ) ;
	int SendData ( char *message ) ;
	//
	
	bool paused ; // whether moves have to wait / game is paused
	
	bool quit ; // whether to quit
	bool quit_state ; // already done?
	bool already_freed ; // already freed my data?
} ;

#endif
