#ifndef __PAX_H
#define __PAX_H

#include "definitions.h"

// theme engine

class Pax
{
	public:
	enum
	{
		object_type_none,
		object_type_var,
		object_type_string,
		object_type_vault,
		object_type_node,
		object_type_sprite,
		object_type_font,
		object_type_textbox,
		object_type_music,
		object_type_rain
	} ;
	
	// object structure
	struct object_item
	{
		unsigned int type                ;
		char         name        [ 256 ] ;
		void         *data               ;
		bool         writeable           ;
		char         parent      [ 256 ] ;
		GlDynArray < std::string > *child_list ;
		char         declared_as [ 256 ] ;
	} ;
	
	void Init ( char *filename, bool new_scan_mode = false ) ;
	bool ReadPax ( char *pax_filename, bool be_quiet = true ) ;
	
	char *GetPkgDirname () ;
	char *GetPkgFilename () ;
	
	char *StripHeadAndTail ( char *text ) ;
	char *StripMarks ( char *text ) ;

	object_item *GetDeclaredObjectFromList ( char *object_name ) ;
	object_item *GetObjectFromList ( char *object_name ) ;
	KrImNode *GetNode ( object_item *item, char *object_name ) ;
	
	int   GetIntVar ( char *object_name ) ;
	char *GetStringVar ( char *object_name ) ;
	char  GetCharFromStringVar ( char *object_name, int pos = 0 ) ;
	GlDynArray < std::string > *GetArgList ( char *string ) ;
	
	int CalcValueFromString ( char *string ) ;
	
	void DrawUserDefinedObjects () ;
	
	void ExecuteScript ( char *scriptname ) ;
	
	void FreeObjects () ;
	
	private:
	bool be_verbose, be_strict, scan_mode ;
	GlDynArray < object_item > *object_list ;
	char pkg_dir[4096], pkg_filename[4096] ;
	
	GlDynArray < std::string > *script_start    ;
	GlDynArray < std::string > *script_ready    ;
	GlDynArray < std::string > *script_go       ;
	GlDynArray < std::string > *script_pause    ;
	GlDynArray < std::string > *script_unpause  ;
	GlDynArray < std::string > *script_quitting ;
	GlDynArray < std::string > *script_quit     ;
	GlDynArray < std::string > *script_event    ;
	
	bool ObjectListNewEntryFull ( unsigned int type, char *name,
									void *data, bool writeable,
									bool auto_declare = FALSE ) ;
	void ObjectListAddChildToParent ( char *child, char *parent,
										bool set_parent_var = TRUE ) ;
	
	char *GetObjectName ( char *line ) ;
	char *GetObjectAction ( char *line ) ;
	GlDynArray < std::string > *GetObjectArgList ( char *line ) ;
	
	int FindObjectInList ( char *object_name ) ; // failed = -1 ; otherwise pos
	
	void FreeObject ( object_item *object ) ;

	char *CalcString ( char *string ) ;
	void InterpretLine ( char *object_name, char *object_action,
							GlDynArray < std::string > *object_arglist ) ;
} ;

#endif
