#ifndef __SPRITE_H
#define __SPRITE_H

#include "definitions.h"

#include "audio.h"

#include "rain.h"

class Sprite
{
	public:
	Sprite () ;
	
	void Init ( KrResourceVault *Vault, char *name, KrImNode *node = NULL,
				int cached_scaling = AUTO, int rain_speed_x = 0, int rain_speed_y = 1 ) ;
	
	void SetXY ( int x, int y ) ;
	void SetDestXY ( int x, int y ) ;
	void SetX_Center ( int pseudo_width = 0 ) ;
	void SetY_Center ( int pseudo_height = 0 ) ;
	void SetXY_Center ( int pseudo_width = 0, int pseudo_height = 0 ) ;
	void SetXY_LeftOf ( int x ) ;
	void SetXY_RightOf ( int x ) ;
	void SetXY_AboveOf ( int y ) ;
	void SetXY_BelowOf ( int y ) ;
	void SetZ ( int z ) ;
	void SetZDepthPos ( int z ) ;
	void SetBounce ( int x_speed, int y_speed ) ;
	void SetBounceList ( GlDynArray < Sprite* > *new_bounce_list ) ;
	
	void SetXSpeed  ( int value ) ;
	void SetYSpeed  ( int value ) ;
	void SetXYSpeed ( int value ) ;
	void SetXAccelFactor  ( int value ) ;
	void SetYAccelFactor  ( int value ) ;
	void SetXYAccelFactor ( int value ) ;
	
	void SetFrames ( int pos, int frames, int direction, bool bounce, bool once = FALSE ) ;
	
	void SetDelays ( int delay = 0, int delay_counter = 0,
										int pause_interval = 0,
										int pause_interval_counter = 0,
										bool pause_interval_randomness = FALSE,
										int max_pause_interval = 0 ) ;

	void SetExplosionResource ( char *name ) ;
	void SetExplosionColor ( U8 red, U8 green, U8 blue ) ;
	void SetTint ( U8 red, U8 green, U8 blue ) ;
	void Brighten ( U8 value ) ;
	void SetDestBrighten ( U8 value ) ;
	void Darken ( U8 value ) ;
	void SetDestDarken ( U8 value ) ;
	
	void SetFadeSpeed ( U8 value ) ;
	U8   GetFadeSpeed () ;
	
	void SetAlpha ( int new_alpha ) ;
	void SetDestAlpha ( int new_alpha ) ;
	int GetAlpha () ;
	
	KrSprite *GetSprite () ;
	KrImNode *FindNodeByName ( char *name ) ;
	
	int GetXPos () ;
	int GetYPos () ;
	int GetWidth () ;
	int GetHeight () ;
	KrSprite *GetParent () ;
	
	void InitRain ( KrImNode *node, int speed_x = 0, int speed_y = 1 ) ;
	void SetRainCreationArea ( int x = AUTO, int y = AUTO,
								int w = AUTO, int h = AUTO ) ;
	
	int ConvertXPos ( KrImNode *src_base, int x ) ;
	int ConvertYPos ( KrImNode *src_base, int y ) ;
	
	void HandOverToBaseAt ( KrImNode *new_base, int x, int y ) ;
	
	void SlideInFrom ( int x, int y ) ;
	void SlideOutTo ( int x, int y ) ;
	void DoSlide () ;
	bool SlideFinishedX () ;
	bool SlideFinishedY () ;
	bool SlideFinishedZ () ;
	bool SlideFinished () ;
	
	void PlaySoundAtFrame ( char *filename, int framenumber ) ;
	void SetMoveSound ( char *filename, int volume = 128 ) ;
	void SetStopSound ( char *filename, int volume = 128 ) ;
	void SetCreationSound ( char *filename, int volume = 128 ) ;
	void SetAccelSound ( char *filename, int volume = 128 ) ;
	void SetBounceSound ( char *filename, int volume = 128 ) ;
	
	void SetPanning ( Audio_Sound sound_struct ) ;
	void PlayMoveSound ( int channel = -1, int times = 1 ) ;
	void PlayStopSound ( int channel = -1, int times = 1 ) ;
	void PlayExtraSound ( int channel = -1, int times = 1 ) ;
	void PlayExplodeSound ( int channel = -1, int times = 1 ) ;
	void PlayCreationSound ( int channel = -1, int times = 1 ) ;
	void PlayAccelSound ( int channel = -1, int times = 1 ) ;
	void PlayBounceSound ( int channel = -1, int times = 1 ) ;
	
	void Glow ( bool activate ) ;
	void Twinkling ( bool activate, int new_max_drops = -1,
						int new_lifetime = -1 ) ;
	void SetTwinklingSpeed ( int speed_x, int speed_y ) ;
	
	void Jump ( int new_height ) ;
	
	void Explode ( unsigned int wait_time = 0, unsigned int speed = 0 ) ;
	void ExplodePhaseTwo () ;
	
	void PauseAnim () ;
	void ContinueAnim () ;
	
	void Draw () ;

	bool GetIsAlreadyDrawn () ;
	bool GetIsQuitting () ;
	bool GetIsDone () ;
	void Quit () ;
	
	void Free () ;

	private:
	// Interpolated point
	typedef struct ipoint
	{
		// From the last logic frame:
		float   ox, oy;         // Position
		
		// From the last/current rendered frame:
		float   gx, gy;         // Interpolated position
	} ipoint;

	ipoint anim_ipos ;
	void SetIPos ( int x, int y ) ;
	
	bool using_opengl ;
	bool use_cached_scaling ;

	KrRect anim_rect ;
	
	KrResourceVault *base_vault ;
	KrSpriteResource *animRes ;
	KrSprite *anim ;
	
	KrSprite *parent_node ;

	int anim_zdepth_pos ;
	int anim_x_pos, anim_dest_x_pos ;
	int anim_y_pos, anim_dest_y_pos ;
	int anim_x_speed, anim_y_speed, anim_xy_speed ;
	int anim_x_accel_factor, anim_y_accel_factor, anim_xy_accel_factor ;
	double anim_z_pos, anim_dest_z_pos, anim_z_speed, anim_z_accel_factor ;
	int anim_bounce_x_mod, anim_bounce_x_speed ;
	int anim_bounce_y_mod, anim_bounce_y_speed ;
	GlDynArray < Sprite* > *anim_bounce_list ;
	int anim_w ;
	int anim_h ;
	int anim_cur_frame ;
	int anim_frames ;
	int anim_direction ;
	bool anim_bounce ;
	bool anim_play_once ;
	int anim_delay ;
	int anim_delay_counter ;
	int anim_pause_interval ;
	int anim_pause_interval_counter ;
	int anim_pause_interval_randomness ;
	int anim_max_pause_interval ;
	
	float step_x, step_y ;
	float slide_speed ;
	int anim_x_pos_mod ;
	int anim_y_pos_mod ;

	int current_brighten, dest_brighten ;
	int current_darken, dest_darken ;
	int current_alpha, dest_alpha ;
	U8  fade_speed ;
	
	bool already_drawn ;

	bool glow_activated ;
	int glow_direction ;
	int glow_value ;
	int glow_counter ;

	int jump_height ;
	int jump_pos ;
	int jump_factor ;

	Audio_Sound move_sound ;
	bool move_sound_enabled ;
	bool move_sound_play ;
	bool move_sound_started ;
	int move_sound_channel ;
	
	Audio_Sound stop_sound ;
	bool stop_sound_enabled ;
	bool stop_sound_play ;
	bool stop_sound_started ;
	int stop_sound_channel ;
	
	Audio_Sound extra_sound ;
	bool extra_sound_enabled ;
	bool extra_sound_play ;
	bool extra_sound_started ;
	int extra_sound_framenumber ;
	int extra_sound_channel ;
	
	bool exploding ; // finishing move ;)
	char explosion_resource[256] ;
	Audio_Sound explode_sound ;
	U8 explosion_tint_red ;
	U8 explosion_tint_green ;
	U8 explosion_tint_blue ;
	bool pre_explosion_phase_done ;
	int pre_explosion_phase_pos ;
	int pre_explosion_speed ;
	int explosion_sound_channel ;
	
	Audio_Sound creation_sound ;
	bool creation_sound_enabled ;
	bool creation_sound_started ;
	int creation_sound_channel ;
	
	Audio_Sound accel_sound ;
	bool accel_sound_enabled ;
	bool accel_sound_started ;
	int accel_sound_channel ;
	
	Audio_Sound bump_sound ;
	bool bump_sound_enabled ;
	bool bump_sound_started ;
	int bump_sound_channel ;
	
	// this will be our star-rain when glowing
	Rain rain ;
	int rain_x, rain_y, rain_w, rain_h ;
	bool twinkling_activated ;
	
	bool anim_paused ;
	
	bool quit ; // wheter to quit
	bool quit_state ; // already done quitting?
	bool already_freed ; // has all my data been erased already?
} ;

#endif
