#ifndef __video_h
#define __video_h

#include "definitions.h"

#include "video_overlay.h"

#include "c1_layer.h"
#include "fps.h"
#include "osd.h"
#include "gamescreen.h"

class Video
{
	public:
	enum
	{
		RESIZE_MODE_NONE,
		RESIZE_MODE_AUTO,
		RESIZE_MODE_OVERLAY,
		RESIZE_MODE_NUM
	} ;
	
	void Init () ;
	
	void Setup () ;
	
	KrEngine *GetEngine () ;
	
	int GetDefaultDepth () ;
	bool GetForceSoftware () ;
	bool GetUsingOpenGL () ;
	bool GetUseCachedScaling () ;
	bool GetDisplayFramerate () ;
	bool GetFramerateReached () ;
	int  GetFramesLeft () ;

	int GetLogicFrames () ;
	int GetRenderedFrames () ;
	
	int GetScreenX () ;
	int GetScreenY () ;
	int GetScreenWidth () ;
	int GetScreenHeight () ;
	double GetZoomX () ;
	double GetZoomY () ;
	SDL_Surface *GetScreenSurface () ;
	Uint32 GetOverlayMode () ;
	bool GetReactOnFocusLost () ;
	bool GetC1ModeActive () ;
	
	void SetFullscreen ( bool value ) ;
	void SetForceOpenGL ( bool value ) ;
	void SetForceSoftware ( bool value ) ;
	void SetDefaultDepth ( int value ) ;
	void SetForceDepth ( bool value ) ;
	void SetWindowResizeMode ( int value ) ;
	void SetUseCachedScaling ( int value ) ;
	void SetOverlayMode ( Uint32 value ) ;
	void SetReactOnFocusLost ( bool value ) ;
	void SetDisplayFramerate ( bool value ) ;
	void SetPausePossible ( bool value ) ;
	
	void SetClearBackground ( bool choice ) ;
	
	void SetDisplayC1Layer ( bool value ) ;
	
	void  SetTime ( float time ) ;
	float GetTime () ;
	
	void SetMouseVisible ( bool value ) ;
	void ShowMouse () ;
	void HideMouse () ;
	
	void Everytime ( SDL_Event event ) ;
	void ToggleFullscreen () ;
	void DumpFPS () ;
	void PrintFPSStatistics () ;
	
	void DisplayOSDMessage ( char *message ) ;
	void HoldOSDMessage () ;
	void UnholdOSDMessage () ;
	bool OSDMessageIsHold () ;
	
	void ResizeVideo ( int w, int h ) ;
	void Update ( bool &pause, bool &toggle_pause_by_user, bool &focus_lost,
					int do_update = AUTO, int frames_until_update = 0 ) ;
	void UpdateDisplayOnly () ;
	
	void QuitGameScreen () ;

	bool GetIsQuitting () ;
	bool GetIsDone () ;
	
	void Quit () ;
	
	private:
	KrEngine *engine ;
	
	Video_Overlay video_overlay ;

	bool display_framerate ;
	bool react_on_focus_lost ;

	int logic_frames ; // count logically rendered frames
	int rendered_frames ; // count visual updates
	
	enum
	{
		RENDER_SPEED_SMOOTH,	// smooth movements
		RENDER_SPEED_NORMAL,	// lose up to 1 frames
		RENDER_SPEED_FAST,	// lose up to 10% of current framerate
		RENDER_SPEED_FASTEST	// lose up to 15% of current framerate
	} ;
	
	int render_speed ;
	
	// whether this seems to be a vaio-c1 or similar model ( 1024x480 )
	bool c1_mode ;
	
	// RESIZE_MODE_NONE is the most direct rendering possible ( fastest )
	int window_resize_mode ;
	
	// which color-depth do we prefer? kyra prefers 32bit !
	int default_depth ;
	
	// this caches scaled sprites ( needs much memory!! )
	bool use_cached_scaling ;
	
	// this will be discovered later.... shouldn't be changed!
	bool using_opengl ;
	
	bool fullscreen_activated ;
	
	SDL_Surface *screen ;
	U32 flags, gl_flags ;
	bool force_opengl ;
	bool force_software ;
	bool force_depth ;
	
	// background color
	bool fill_background ;
	KrRGBA background_color ;
	
	// time in frames from main ()
	float current_time ;
	
	// Vaio C1 layer
	C1_Layer c1_layer ;
	
	// FPS
	FPS fps ;
	int frames_left ; // how many frames until next visual update
	
	// OSD
	OSD osd ;
	
	// if wanted, these represent our zoom factors for the main window
	double zoom_x, zoom_y ;
	
	// when window is resizable we won't draw every frame to the screen...
	int frame_skipper_max, frame_skipper ;
	
	GameScreen gamescreen ;

	int depth ;

	int screenx ;
	int screeny ;
	int screen_width ;
	int screen_height ;
	SDL_Surface *virtual_display ;
	SDL_Overlay *overlay ;
	
	// whether one can pause the game or not
	bool pause_possible ;
	
	// should our mouse-cursor be visible?
	bool mouse_visible ;

	bool done ;
} ;

#endif
